#include <WaveRP.h>
#include "freeRam.h"
#include <SdFat.h>
#include <Sd2Card.h>
#include "PgmPrint.h"
#include <ctype.h>

// global variables
Sd2Card card;           // SD/SDHC card with support for version 2.00 features
SdVolume volume;           // FAT16 or FAT32 volume
SdFile root;            // volume's root directory
SdFile file;            // current file
SdFile soundfile;        // the file to play
WaveRP wave;

int SCAN_ENTER = 0x5a; int SCAN_BREAK = 0xf0;
int breakActive = 0;
int clockPin = 15; // Clock is only output. 
int dataPin = 14; // The data pin is bi-directional  But at the moment we are only interested in receiving   
int ledPin = 6;  // When a SCAN_ENTER scancode is received the LED blink
int randPin = 6; // this is the analog pin used for random numbers
int clockValue = 0; byte dataValue;
byte scanCodes[50] = {0x70, 0x69, 0x72, 0x7a, 0x6b, 0x73, 0x74, 0x6c, 0x75, 0x7d, 0x1c, 0x32, 0x21, 0x23, 0x24, 0x2b, 0x34, 0x33, 0x43, 0x3b, 0x42, 0x4b, 0x3a, 0x31, 0x44, 0x4d, 0x15, 0x2d, 0x1b, 0x2c, 0x3c, 0x2a, 0x1d, 0x22, 0x35, 0x1a, 0x16, 0x1e, 0x26, 0x25, 0x2e, 0x36, 0x3d, 0x3e, 0x46, 0x45, 0x29, 0x41, 0x49, 0x4a};
;
char characters[50] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', ' ', ',', '.', '/'};
int quantityCodes = 50;
int bufferLength = 64;
char buffer[64];      // serial input buffer
int bufferPos = 0; 
const char table[65] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+-";
char* beeps[] = {"meow01.wav","meow10.wav","meow11.wav"};
const byte numBeeps = 3; // 

uint8_t bufferidx = 0;
uint32_t tmp;

/*
 * print PROGMEM string
 */
 void printProgStr(const char *str)
{
  uint8_t b;
  while((b = pgm_read_byte(str++)))Serial.print(b);
}
/*
 * print PROGMEM string and CR LF
 */
void printProgStrln(const char *str)
{
  printProgStr(str);
  Serial.println();
}
// macros to use PSTR
#define putstring(str) printProgStr(PSTR(str))
#define putstring_nl(str) printProgStrln(PSTR(str))

void playFile(char * name) {
  if (!soundfile.open(root, name)) {
    PgmPrint("Can't open: ");
    Serial.println(name);
    return;
  }
  if (!wave.play(soundfile)) {
    PgmPrint("Can't play: ");Serial.println(name);
    soundfile.close();
    return;
  }
  PgmPrint("Playing: ");Serial.print(name);
  digitalWrite(ledPin, HIGH);

  while (wave.isPlaying()) {
    // do nothing
    delay(50);
    Serial.print(".");
  }
  soundfile.close();
  digitalWrite(ledPin, LOW);
  Serial.println("Done playing, soundfile closed.");
}

uint8_t i;

void setup()                    // run once, when the sketch starts
{
  Serial.begin(9600);
  Serial.println("BarCodeLogger");
  pinMode(dataPin, INPUT);  // data pin for cuecat                                             
  pinMode(clockPin, INPUT); // clock pin for cuecat
  pinMode(ledPin, OUTPUT);
  randomSeed(analogRead(randPin)); 
  if (!card.init()) {
    Serial.println("Card init. failed!"); 
  }
  if (!volume.init(card)) {
    Serial.println("No partition!"); 
  }
  if (!root.openRoot(volume)) {
        Serial.println("Can't open root dir"); 
  }
  strcpy(buffer, "BARLOG00.TXT");
  for (i = 0; i < 100; i++) {
    buffer[6] = '0' + i/10;
    buffer[7] = '0' + i%10;
    if (file.open(root, buffer)) { // if able to open the file
      Serial.print("Opening file:");Serial.println(buffer);
      if(file.fileSize()==0) { // we only want empty files
        break;
      } else {
        Serial.println("  File already has data, closing");
        file.close(); // close the file because it already has data in it, then continue on
      }
    } else {
      if (file.create(root, buffer)) break;
    }
  }
  
  if(!file.isOpen()) {
    Serial.print("couldnt create "); Serial.println(buffer);
  }
  Serial.print("writing to "); Serial.println(buffer);
  Serial.println("ready!");

  Serial.println("Incomming Data");
  /* 
  // clear print error
  file.writeByteError = 0;
  file.println("Incomming Data");
  if (file.writeByteError || !file.sync()) {
    putstring_nl("can't write header!");
  }
  */
  playFile("start.wav");
}

void loop() { // main program
  dataValue = dataRead();
  if (dataValue == SCAN_BREAK) {                                         
    breakActive = 1;                                                     
  }
  // Translate the scan codes to numbers                                 
  // If there is a match, store it to the buffer                         
  for (int i = 0; i < quantityCodes; i++) {      	                       
    byte temp = scanCodes[i];                                            
    if(temp == dataValue){                                               
      if(!breakActive == 1){                                             
        buffer[bufferPos] = characters[i];                               
        bufferPos++;  
      } // end of breakActive                                                                 
    } // end of "valid character"
  } // end of quantityCodes check                                                                     

  if(dataValue == SCAN_ENTER) {                                            
    Serial.print("Writing: ");
    Serial.println(buffer);
    file.writeByteError = 0;
    file.println(buffer);
    if (file.writeByteError || !file.sync()) { // can't write for some reason
      Serial.print("can't write data!");
      return;
    }
    int i=0;																		                         
    if (buffer[i] != 0) {                                                
      while(buffer[i] != 0) {                                            
        buffer[i] = 0;                                                   
         i++;                                                             
      }                                                                  
    } // end of clear buffer loop
    bufferPos = 0;
    digitalWrite(ledPin, HIGH);
    delay(300);
    digitalWrite(ledPin, LOW);
    playFile(beeps[random(numBeeps)]);
  } // end of "SCAN_ENTER" check
  // Reset the SCAN_BREAK state if the byte was a normal one             
  if(dataValue != SCAN_BREAK){                                           
    breakActive = 0;                                                     
  }                                                                      
  dataValue = 0;
} // end of loop()

int dataRead() {
  byte val = 0;                                                          
  // Skip start state and start bit                                      
  while (digitalRead(clockPin));  // Wait for LOW.                       
  // clock is high when idle                                             
  while (!digitalRead(clockPin)); // Wait for HIGH.                      
  while (digitalRead(clockPin));  // Wait for LOW.                       
  for (int offset = 0; offset < 8; offset++) {                           
    while (digitalRead(clockPin));         // Wait for LOW               
    val |= digitalRead(dataPin) << offset; // Add to byte                
    while (!digitalRead(clockPin));        // Wait for HIGH              
  }                                                                      
// Skipping parity and stop bits down here.                            
  while (digitalRead(clockPin));           // Wait for LOW.              
  while (!digitalRead(clockPin));          // Wait for HIGH.             
  while (digitalRead(clockPin));           // Wait for LOW.              
  while (!digitalRead(clockPin));          // Wait for HIGH.             
  return val;                                                            
}

